/*  $Id: WeakHashtable.java 1035 2010-06-15 13:08:33Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import java.util.Vector;


/** 
 * Zip file creation utlity.
 */
public class Archive implements Runnable
{
	/** target zip file path */
	private String _archiveFilePath;
	/** File list to be include in the zip archive */
	private Vector<File> _fileList;
	/** Overwrite strategy. */
	private int _overwriteMode=OM_STOP;
	/** 
	 * Don't overwrite.
	 */
	public final static int OM_STOP=0;
	/** 
	 * Backup before overwrite.
	 */
	public final static int OM_BACKUP=1;
	/** 
	 * Overwrite without backup.
	 */ 
	public final static int OM_IGNORE=2;
	/** Error handler */
	private ErrorHandler _errHandler;
	/** 
	 * Path prefix to be removed in archive file index. 
	 */
	private String _removePrefix="/";
	
	/** 
	 * Create new archive command.
	 *
	 * @param archivePath target file path.
	 * @param prefix leading prefix to remove from file paths index
	 * @param overwriteMode overwrite mode, see OM_* constants.
	 */ 
	public Archive(String archivePath,String prefix,int overwriteMode)
	{
		_archiveFilePath=archivePath;
		_overwriteMode=overwriteMode;
		_errHandler=App.get().getErrHandler();
		_removePrefix=(new File(prefix)).getPath()+File.separatorChar;
		_fileList=new Vector<File>();
	}

	/** 
	 * Change error handler.
	 * @param errHandler new error handler.
	 */ 
	public void setErrHandler(ErrorHandler errHandler) 
	{
		_errHandler=errHandler;
	}

	/** 
	 * Add a file to the archive.
	 * Registration only. The file is really inserted by the run method.
	 * @param path path of file to add.
	 */ 
	public void addFile(String path) {
		addFile(new File(path));
	}
	/** 
	 * Add a file to the archive.
	 * Registration only. The file is really inserted by the run method.
	 * @param f file to add.
	 */ 
	public void addFile(File f)
	{
		if(f.exists())
		{
			_fileList.add(f);
		}
	}

	/** 
	 * Ajouter un repertoire a l'archive
	 * 
	 * L'ajout n'est pas immédiat, il ne sera effectif que lors de la
	 * création de l'archive par la méthode Run.
	 * @param path chemin du repertoire
	 * @param recursive true : ajoute le répertoire ainsi que tout
	 * son contenu, false : ajoute le répertoir uniquement (qui sera vide 
	 * si aucun contenu n'est ajouté ultérieurement)
	 */ 
	public void addDir(String path,boolean recursive) {
		addDir(new File(path),recursive);
	}
	public void addDir(File f,boolean recursive)
	{
		if (f.exists()&&f.isDirectory())
		{
			_fileList.add(f);
			if (recursive)
			{
				for (File content : f.listFiles())
				{
					if (content.isDirectory()) {
						addDir(content,recursive);
					}
					else {
						addFile(content);
					}
				}	
			}
		}
	}

	/** 
	 * Effectuer la compression et l'écriture de l'archive.
	 *
	 * <p>Au préalable il faut bien entendu au moins définir un fichier
	 * à inclure dans l'archive (méthodes AddFile() ou AddDir()).</p>
	 */ 
	public void run()
	{
		File f=new File(_archiveFilePath);
		if (f.exists())	{
			switch (_overwriteMode) 
			{
				case OM_BACKUP:
					f.renameTo(new File(_archiveFilePath+".bak"));
					break;
				case OM_IGNORE:
					f.delete();
					break;
				default:
					Error err=new Error(getClass().getName()+".create",
                                                            Error.ELevel.ERR);
					err.addArg(_archiveFilePath);
					_errHandler.handle(err);
					return;
			}
		}
		if (jc.O.DEBUG) {
			App.get().log().debug("Create zip file: "+_archiveFilePath);
		}
		try {
			ZipOutputStream zip=new ZipOutputStream(
                                new FileOutputStream(_archiveFilePath));
			for(File fi : _fileList)
			{
				String entryPath=fi.getPath();
				if (entryPath.startsWith(_removePrefix)) {
					entryPath=entryPath.substring(_removePrefix.length());
				}
				if (jc.O.DEBUG) {
					App.get().log().debug("Adding zip file entry: "+
                            fi.getPath()+" as "+entryPath);
				}	
				ZipEntry entry=new ZipEntry(entryPath);
				zip.putNextEntry(entry);
				if (!fi.isDirectory()) {
					FileInputStream in=new FileInputStream(fi);
					byte[] buffer=new byte[4096];
					int len=in.read(buffer);
					while (len>0) {
						zip.write(buffer, 0, len);
						len=in.read(buffer);
					}
					in.close();
				}	
				zip.closeEntry();
			}
			zip.close();
		}
		catch (FileNotFoundException fex) {
			Error err=new Error(getClass().getName()+".open",
                                            Error.ELevel.ERR,fex);
			err.addArg(_archiveFilePath);
			_errHandler.handle(err);
		}
		catch (IOException ioex) {
			Error err=new Error(getClass().getName()+".write",
                                            Error.ELevel.ERR,ioex);
			err.addArg(_archiveFilePath);
			_errHandler.handle(err);
		}
		if (jc.O.DEBUG) {
			App.get().log().debug("Create zip file completed: "+
                                                        _archiveFilePath);
		}
	}
}
